@@ -11,8 +11,9 @@ our @ISA = ('Database::Abstraction');
 # Doesn't ignore lines starting with '#' as it's not treated like a CSV file
 sub _open {
 	my $self = shift;
+	my %args = (ref($_[0]) eq 'HASH') ? %{$_[0]} : @_;
 
-	return $self->SUPER::_open(sep_char => ':', column_names => ['name', 'password', 'uid', 'gid', 'gecos', 'home', 'shell']);
+	return $self->SUPER::_open(sep_char => ':', column_names => ['name', 'password', 'uid', 'gid', 'gecos', 'home', 'shell'], %args);
 }
 
 1;
